#include "dictselection.h"
#include "dictsettings.h"

#include <KConfig>
#include <QApplication>
#include <QClipboard>


DictSelection::DictSelection()
{
	timer = 0;
	local = remote = desktop = false;
	minLen = maxLen = 0;
	clipboard = QApplication::clipboard();
}

void DictSelection::setup(const DictConf &conf)
{
	local = conf.localYank;
	remote = conf.remoteYank;
	desktop = conf.desktopYank;

	minLen = conf.yankMin;
	maxLen = conf.yankMax;

	set(false);
	set(conf.yank);
}

void DictSelection::set(bool on)
{
	if (state == on)
		return;
	else
		state = on;

	if (on) {
		if (desktop) setTimeout(1000);
		connect(clipboard, SIGNAL(selectionChanged()),
			SLOT(selection()));
	} else {
		setTimeout(0);
		clipboard->disconnect(this);
	}
}

static QString lastSelection;

QString DictSelection::currentSelection()
{
	return clipboard->text(QClipboard::Selection).left(maxLen);
}

void DictSelection::selection()
{
	lastSelection = currentSelection();

	bool owner = clipboard->ownsSelection();
	if (!local && owner)
		return;
	if (!remote && !owner)
		return;

	if (lastSelection.length() < minLen
	|| lastSelection.length() >= maxLen)
		return;

	emit selectionChanged(lastSelection);
}

void DictSelection::setTimeout(int timeout)
{
	if (timer)
		killTimer(timer);
	if (timeout) {
		lastSelection = currentSelection();
		timer = startTimer(timeout);
	}
}

void DictSelection::timerEvent(QTimerEvent *event)
{
	if (lastSelection != currentSelection()) {
		lastSelection = currentSelection();
		if (lastSelection.length() >= minLen
		&& lastSelection.length() < maxLen)
			emit selectionChanged(lastSelection);
	}
}
