#include "dicttracker.h"
#include "dictpage.h"


DictTracker::DictTracker(Dict *d, DictPage *w, DictPage *m)
	: dict(d), wordPage(w), matchPage(m)
{
	clear();
	correct = false;
	setDefaultDatabase(0);
	database << defaultDatabase;
}

void DictTracker::define(const QString &text)
{
	query = text;
	reset();
	defining = true;
	dbcount = database.size();
	Q_ASSERT(dbcount);
	emit loading(true);

	foreach(QString dbase, database)
		dict->define(query, dbase);
}

void DictTracker::match(const QString &text, const QString &strategy)
{
	query = text;
	matching = true;
	dbcount = database.size();
	Q_ASSERT(dbcount);
	QString strat = (!strategy.isEmpty()) ? strategy : Dict::DefaultStrategy;
	emit loading(true);

	foreach(QString dbase, database)
		dict->match(query, dbase, strat);
}

void DictTracker::status(const QString &text)
{
	query = text;
	emit loading(true);
	dict->status();
}

void DictTracker::info(const QString &text, const QString &info)
{
	query = text;
	emit loading(true);
	if (info == "status")
		dict->status();
	else {
		dbinit = stratinit = false;
		dict->info(info);
	}
}

void DictTracker::cancel()
{
	dict->cancel();
	clear();
}

void DictTracker::clear()
{
	reset();
	defining = matching = correcting = appending = false;
	emit loading(false);
}

void DictTracker::setDatabase(const QStringList &list)
{
	database = (list.size()) ? list : QStringList(defaultDatabase);
}

void DictTracker::setDefaultDatabase(int defaultSearch)
{
	defaultDatabase = (defaultSearch)
		? Dict::AllDatabases : Dict::DefaultDatabase;

	if (database.size() == 1 && (database[0] == ((defaultSearch)
		? Dict::DefaultDatabase : Dict::AllDatabases)))
		database[0] = defaultDatabase;
}

void DictTracker::init()
{
	clear();
	dbinit = stratinit = true;
	dict->info("databases");
	dict->info("strategies");
// 	emit loading(true);
}

void DictTracker::response(const DictResponse &response)
{
	DictPage *page = wordPage;

	switch (response.type) {
	case DictResponse::Words:
		emit words();
		break;
	case DictResponse::Matches:
		page->correcting(correcting);
		if (correcting)
			emit corrections();
		else if (matchPage) {
			page = matchPage;
			emit matches();
		}
		break;
	case DictResponse::NoMatch:
		if (++misses < dbcount) {
			if (count() >= dbcount)
				clear();
			return;
		} else if (!correcting) {
			page->nomatches(query, defining);
			if (correct) {
				reset();
				correcting = true;
				match(query, Dict::DefaultStrategy);
				return;
			}
		} else
			page->corrections(false);
		break;
	case DictResponse::DatabaseList:
		emit databases(response.list);
		if (dbinit) {
			dbinit = false;
			return;
		}
		break;
	case DictResponse::StrategyList:
		emit strategies(response.list);
		if (stratinit) {
// 			emit loading(false);
			stratinit = false;
			return;
		}
		break;
	case DictResponse::ServerInfo:
	case DictResponse::DatabaseInfo:
		break;
	case DictResponse::StatusMessage:
		page->status(dict);
		break;
	case DictResponse::AuthOK:
		init();
		break;
	default:
		qDebug() << "unhandled type" << response.type;
	}

	page->setTitle(query);
	page->set(response, appending);
	appending = true;

	if (response.type == DictResponse::Words
	|| response.type == DictResponse::Matches) {
		if (response.done) ++hits;
		if (count() >= dbcount)
			clear();
	} else
		clear();
}
