#ifndef DICTTRACKER_H
#define DICTTRACKER_H

#include "dict.h"

class DictPage;

class DictTracker : public QObject
{
	Q_OBJECT

public:
	DictTracker(Dict *dict, DictPage *word, DictPage *match = 0);

	void define(const QString &query);
	void match(const QString &query, const QString &strategy);
	void info(const QString &query, const QString &text);
	void status(const QString &query);
	void cancel();
	void init();
	void clear();

	void setCorrect(bool on) { correct = on; };
	void setMatchPage(DictPage *match) { matchPage = match; };
	QStringList databases() const { return database; };

private:
	bool dbinit, stratinit;
	bool defining, matching, correcting, appending;
	int hits, misses, dbcount;
	void reset() { hits = misses = 0; };
	int count() { return hits + misses; };

	Dict *dict;
	DictPage *wordPage, *matchPage;
	bool correct;
	QString query;
	QStringList database;
	QString defaultDatabase;

public Q_SLOTS:
	void response(const DictResponse &response);
	void setDatabase(const QStringList &database);
	void setDefaultDatabase(int defaultSearch);

Q_SIGNALS:
	void words();
	void matches();
	void corrections();
	void loading(bool);
	void databases(const DictList &list);
	void strategies(const DictList &list);
};

#endif
